import os
from typing import ByteString, Union

from planqk.commons.constants import OUTPUT_DIRECTORY_ENV, DEFAULT_OUTPUT_DIRECTORY
from planqk.commons.file import write_str_to_file, write_blob_to_file


def write_string_output(file_name: str, content: str):
    directory_path = os.environ.get(OUTPUT_DIRECTORY_ENV, DEFAULT_OUTPUT_DIRECTORY)
    write_str_to_file(directory_path, file_name, content)


def write_binary_output(file_name: str, content: Union[bytes, bytearray, ByteString]):
    directory_path = os.environ.get(OUTPUT_DIRECTORY_ENV, DEFAULT_OUTPUT_DIRECTORY)
    write_blob_to_file(directory_path, file_name, content)



import os
from typing import Dict, Any

import yaml


def read_to_dict(file_path: str) -> Dict[str, Any]:
    with open(file_path, "r") as file:
        openapi = yaml.safe_load(file)

    return openapi


def write_dict(directory_path: str, openapi: Dict[str, Any], filename: str = "openapi.yaml"):
    file_path = os.path.join(directory_path, filename)
    with open(file_path, "w") as file:
        yaml.dump(openapi, file, sort_keys=False)



import os
import traceback

from loguru import logger

from planqk.commons import __version__
from planqk.commons.constants import OUTPUT_DIRECTORY_ENV, INPUT_DIRECTORY_ENV, ENTRYPOINT_ENV, DEFAULT_ENTRYPOINT, DEFAULT_INPUT_DIRECTORY, \
    DEFAULT_OUTPUT_DIRECTORY
from planqk.commons.entrypoint import run_entrypoint
from planqk.commons.file import list_directory_files, write_str_to_file
from planqk.commons.json import any_to_json
from planqk.commons.parmeters import files_to_parameters, is_simple_type
from planqk.commons.reflection import resolve_signature


def main() -> int:
    logger.debug(f"planqk-commons Version: {__version__}")

    entrypoint = os.environ.get(ENTRYPOINT_ENV, DEFAULT_ENTRYPOINT)
    entrypoint_signature = resolve_signature(entrypoint)
    logger.debug(f"Entrypoint: {entrypoint}")

    input_directory = os.environ.get(INPUT_DIRECTORY_ENV, DEFAULT_INPUT_DIRECTORY)
    output_directory = os.environ.get(OUTPUT_DIRECTORY_ENV, DEFAULT_OUTPUT_DIRECTORY)

    input_files = list_directory_files(input_directory)
    logger.debug(f"Input files: {list(input_files.values())}")

    input_parameters = files_to_parameters(input_files, entrypoint_signature)

    try:
        return_value = run_entrypoint(entrypoint, input_parameters)
    except Exception as e:
        logger.error(f"{e}")
        traceback.print_exc()
        return 1

    logger.debug(f"Return type: {type(return_value)}")

    string_value = any_to_json(return_value)
    if string_value is None:
        return 0

    if is_simple_type(return_value):
        file_path = write_str_to_file(output_directory, "output.txt", string_value)
    else:
        file_path = write_str_to_file(output_directory, "output.json", string_value)
    logger.debug(f"Output file: {file_path}")

    return 0



import json
from typing import Optional

from pydantic import BaseModel


def any_to_json(data: any) -> Optional[str]:
    if data is None:
        return None

    if issubclass(data.__class__, BaseModel):
        return data.model_dump_json(indent=2)

    return json.dumps(data, default=lambda o: getattr(o, "__dict__", str(o)), indent=2)
